@@ -2,12 +2,16 @@
 #
 # $Id$
 #
+# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+# Use of this source code is governed by a BSD-style license that can be
+# found in the LICENSE file.
 
 """psutil is a module providing convenience functions for managing
-processes in a portable way by using Python.
+processes and gather system information in a portable way by using
+Python.
 """
 
-__version__ = "0.3.0"
+__version__ = "0.4.1"
 version_info = tuple([int(num) for num in __version__.split('.')])
 
 __all__ = [
@@ -24,7 +28,8 @@ __all__ = [
     # functions
     "test", "pid_exists", "get_pid_list", "process_iter", "get_process_list",
     "phymem_usage", "virtmem_usage"
-    "cpu_times", "per_cpu_times", "cpu_percent", "per_cpu_percent"
+    "cpu_times", "per_cpu_times", "cpu_percent", "per_cpu_percent",
+    "network_io_counters", "disk_io_counters",
     ]
 
 import sys
@@ -122,18 +127,6 @@ class Process(object):
     def __repr__(self):
         return "<%s at %s>" % (self.__str__(), id(self))
 
-    def __eq__(self, other):
-        """Test for equality with another Process object based on pid
-        and creation time.
-        """
-        h1 = (self.pid, self.create_time)
-        try:
-            h2 = (other.pid, other.create_time)
-        except AttributeError:
-            return False
-        else:
-            return h1 == h2
-
     @property
     def pid(self):
         """The process pid."""
@@ -355,9 +348,13 @@ class Process(object):
             return 0.0
         # the utilization of a single CPU
         single_cpu_percent = overall_percent * NUM_CPUS
-        # ugly hack to avoid troubles with float precision issues
-        if single_cpu_percent > 100.0:
-            return 100.0
+        # on posix a percentage > 100 is legitimate
+        # http://stackoverflow.com/questions/1032357/comprehending-top-cpu-usage
+        # on windows we use this ugly hack to avoid troubles with float
+        # precision issues
+        if os.name != 'posix':
+            if single_cpu_percent > 100.0:
+                return 100.0
         return round(single_cpu_percent, 1)
 
     def get_cpu_times(self):
@@ -393,23 +390,41 @@ class Process(object):
         """
         return self._platform_impl.get_open_files()
 
-    def get_connections(self):
-        """Return TCP and UPD connections opened by process as a list
-        of namedtuples.
-        On BSD and OSX results for third party processes (!= os.getpid())
-        can differ depending on user privileges.
+    def get_connections(self, kind='inet'):
+        """Return connections opened by process as a list of namedtuples.
+        The kind parameter filters for connections that fit the following
+        criteria:
+
+        Kind Value      Connections using
+        inet            IPv4 and IPv6
+        inet4           IPv4
+        inet6           IPv6
+        tcp             TCP
+        tcp4            TCP over IPv4
+        tcp6            TCP over IPv6
+        udp             UDP
+        udp4            UDP over IPv4
+        udp6            UDP over IPv6
+        all             the sum of all the possible families and protocols
         """
-        return self._platform_impl.get_connections()
+        return self._platform_impl.get_connections(kind)
 
     def is_running(self):
-        """Return whether the current process is running in the current
-        process list.
-        """
+        """Return whether this process is running."""
         try:
-            newproc = Process(self.pid)
-            return self == newproc
+            # Test for equality with another Process object based
+            # on pid and creation time.
+            # This pair is supposed to indentify a Process instance
+            # univocally over the time (the PID alone is not enough as
+            # it might refer to a process which is gone in meantime
+            # and its PID reused by another process).
+            new_self = Process(self.pid)
+            p1 = (self.pid, self.create_time)
+            p2 = (new_self.pid, new_self.create_time)
         except NoSuchProcess:
             return False
+        else:
+            return p1 == p2
 
     def send_signal(self, sig):
         """Send a signal to process (see signal module constants).
@@ -487,6 +502,8 @@ class Process(object):
         """Wait for process to terminate and, if process is a children
         of the current one also return its exit code, else None.
         """
+        if timeout is not None and not timeout >= 0:
+            raise ValueError("timeout must be a positive integer")
         return self._platform_impl.process_wait(timeout)
 
 
@@ -673,6 +690,69 @@ def disk_partitions(all=False):
     """
     return _psplatform.disk_partitions(all)
 
+def network_io_counters(pernic=False):
+    """Return network I/O statistics as a namedtuple including
+    the following attributes:
+
+     - bytes_sent:   number of bytes sent
+     - bytes_recv:   number of bytes received
+     - packets_sent: number of packets sent
+     - packets_recv: number of packets received
+
+    If pernic is True return the same information for every
+    network interface installed on the system as a dictionary
+    with network interface names as the keys and the namedtuple
+    described above as the values.
+    """
+    from psutil._common import ntuple_net_iostat
+    rawdict = _psplatform.network_io_counters()
+    if pernic:
+        for nic, fields in rawdict.iteritems():
+            rawdict[nic] = ntuple_net_iostat(*fields)
+        return rawdict
+    else:
+        bytes_sent, bytes_recv, packets_sent, packets_recv = 0, 0, 0, 0
+        for _, fields in rawdict.iteritems():
+            bytes_sent += fields[0]
+            bytes_recv += fields[1]
+            packets_sent += fields[2]
+            packets_recv += fields[3]
+        return ntuple_net_iostat(bytes_sent, bytes_recv,
+                                 packets_sent, packets_recv)
+
+def disk_io_counters(perdisk=False):
+    """Return system disk I/O statistics as a namedtuple including
+    the following attributes:
+
+     - read_count:  number of reads
+     - write_count: number of writes
+     - read_bytes:  number of bytes read
+     - write_bytes: number of bytes written
+     - read_time:   time spent reading from disk (in milliseconds)
+     - write_time:  time spent writing to disk (in milliseconds)
+
+    If perdisk is True return the same information for every
+    physical disk installed on the system as a dictionary
+    with partition names as the keys and the namedutuple
+    described above as the values.
+    """
+    from psutil._common import ntuple_disk_iostat
+    rawdict = _psplatform.disk_io_counters()
+    if perdisk:
+        for disk, fields in rawdict.iteritems():
+            rawdict[disk] = ntuple_disk_iostat(*fields)
+        return rawdict
+    else:
+        reads, writes, rbytes, wbytes, rtime, wtime = 0, 0, 0, 0, 0, 0
+        for _, fields in rawdict.iteritems():
+            reads += fields[0]
+            writes += fields[1]
+            rbytes += fields[2]
+            wbytes += fields[3]
+            rtime += fields[4]
+            wtime += fields[5]
+        return ntuple_disk_iostat(reads, writes, rbytes, wbytes, rtime, wtime)
+
 
 def _deprecated(replacement):
     # a decorator which can be used to mark functions as deprecated
@@ -754,4 +834,3 @@ def test():
 
 if __name__ == "__main__":
     test()
-
