@@ -2,6 +2,11 @@
 #
 # $Id$
 #
+# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+# Use of this source code is governed by a BSD-style license that can be
+# found in the LICENSE file.
+
+"""Linux platform implementation."""
 
 import os
 import errno
@@ -9,6 +14,7 @@ import socket
 import struct
 import sys
 import base64
+import re
 
 import _psutil_posix
 import _psutil_linux
@@ -35,17 +41,41 @@ def _get_boot_time():
 
 def _get_num_cpus():
     """Return the number of CPUs on the system"""
-    num = 0
-    f = open('/proc/cpuinfo', 'r')
+    # we try to determine num CPUs by using different approaches.
+    # SC_NPROCESSORS_ONLN seems to be the safer and it is also
+    # used by multiprocessing module
     try:
-        for line in f:
-            if line.startswith('processor'):
+        return os.sysconf("SC_NPROCESSORS_ONLN")
+    except ValueError:
+        # as a second fallback we try to parse /proc/cpuinfo
+        num = 0
+        f = open('/proc/cpuinfo', 'r')
+        try:
+            lines = f.readlines()
+        finally:
+            f.close()
+        for line in lines:
+            if line.lower().startswith('processor'):
                 num += 1
-        if num == 0:
-            raise RuntimeError("line not found")
-        return num
-    finally:
-        f.close()
+
+    # unknown format (e.g. amrel/sparc architectures), see:
+    # http://code.google.com/p/psutil/issues/detail?id=200
+    # try to parse /proc/stat as a last resort
+    if num == 0:
+        f = open('/proc/stat', 'r')
+        try:
+            lines = f.readlines()
+        finally:
+            f.close()
+        search = re.compile('cpu\d')
+        for line in lines:
+            line = line.split(' ')[0]
+            if search.match(line):
+                num += 1
+
+    if num == 0:
+        raise RuntimeError("can't determine number of CPUs")
+    return num
 
 
 # Number of clock ticks per second
@@ -93,37 +123,20 @@ def phymem_buffers():
     kernel in bytes.
     This reflects the "buffers" column of free command line utility.
     """
-    f = open('/proc/meminfo', 'r')
-    try:
-        for line in f:
-            if line.startswith('Buffers:'):
-                return int(line.split()[1]) * 1024
-        raise RuntimeError("line not found")
-    finally:
-        f.close()
+    total, free, buffers = _psutil_linux.get_physmem()
+    return buffers
 
 def phymem_usage():
+    """Return physical memory usage statistics as a namedutple including
+    tota, used, free and percent usage.
+    """
     # total, used and free values are matched against free cmdline utility
     # the percentage matches top/htop and gnome-system-monitor
-    f = open('/proc/meminfo', 'r')
-    try:
-        total = free = buffers = cached = None
-        for line in f:
-            if line.startswith('MemTotal:'):
-                total = int(line.split()[1]) * 1024
-            elif line.startswith('MemFree:'):
-                free = int(line.split()[1]) * 1024
-            elif line.startswith('Buffers:'):
-                buffers = int(line.split()[1]) * 1024
-            elif line.startswith('Cached:'):
-                cached = int(line.split()[1]) * 1024
-                break
-        used = total - free
-        percent = usage_percent(total - (free + buffers + cached), total,
-                                _round=1)
-        return ntuple_sysmeminfo(total, used, free, percent)
-    finally:
-        f.close()
+    total, free, buffers = _psutil_linux.get_physmem()
+    cached = cached_phymem()
+    used = total - free
+    percent = usage_percent(total - (free + buffers + cached), total, _round=1)
+    return ntuple_sysmeminfo(total, used, free, percent)
 
 
 def virtmem_usage():
@@ -209,19 +222,82 @@ def disk_partitions(all=False):
 
 get_disk_usage = _psposix.get_disk_usage
 
+
 # --- process functions
 
 def get_pid_list():
     """Returns a list of PIDs currently running on the system."""
     pids = [int(x) for x in os.listdir('/proc') if x.isdigit()]
-    # special case for 0 (kernel process) PID
-    pids.insert(0, 0)
     return pids
 
 def pid_exists(pid):
     """Check For the existence of a unix pid."""
     return _psposix.pid_exists(pid)
 
+def network_io_counters():
+    """Return network I/O statistics for every network interface
+    installed on the system as a dict of raw tuples.
+    """
+    f = open("/proc/net/dev", "r")
+    try:
+        lines = f.readlines()
+    finally:
+        f.close()
+
+    retdict = {}
+    for line in lines[2:]:
+        fields = line.split()
+        name = fields[0][:-1]
+        bytes_recv = int(fields[1])
+        packets_recv = int(fields[2])
+        bytes_sent = int(fields[9])
+        packets_sent = int(fields[10])
+        retdict[name] = (bytes_sent, bytes_recv, packets_sent, packets_recv)
+    return retdict
+
+def disk_io_counters():
+    """Return disk I/O statistics for every disk installed on the
+    system as a dict of raw tuples.
+    """
+    # man iostat states that sectors are equivalent with blocks and
+    # have a size of 512 bytes since 2.4 kernels. This value is
+    # needed to calculate the amount of disk I/O in bytes.
+    SECTOR_SIZE = 512
+
+    # determine partitions we want to look for
+    partitions = []
+    f = open("/proc/partitions", "r")
+    try:
+        lines = f.readlines()[2:]
+    finally:
+        f.close()
+    for line in lines:
+        _, _, _, name = line.split()
+        if name[-1].isdigit():
+            partitions.append(name)
+    #
+    retdict = {}
+    f = open("/proc/diskstats", "r")
+    try:
+        lines = f.readlines()
+    finally:
+        f.close()
+    for line in lines:
+        _, _, name, reads, _, rbytes, rtime, writes, _, wbytes, wtime = \
+            line.split()[:11]
+        if name in partitions:
+            rbytes = int(rbytes) * SECTOR_SIZE
+            wbytes = int(wbytes) * SECTOR_SIZE
+            reads = int(reads)
+            writes = int(writes)
+            # TODO: times are expressed in milliseconds while OSX/BSD has
+            # these expressed in nanoseconds; figure this out.
+            rtime = int(rtime)
+            wtime = int(wtime)
+            retdict[name] = (reads, writes, rbytes, wbytes, rtime, wtime)
+    return retdict
+
+
 # taken from /fs/proc/array.c
 _status_map = {"R" : STATUS_RUNNING,
                "S" : STATUS_SLEEPING,
@@ -266,8 +342,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_process_name(self):
-        if self.pid == 0:
-            return 'sched'  # special case for kernel process
         f = open("/proc/%s/stat" % self.pid)
         try:
             name = f.read().split(' ')[1].replace('(', '').replace(')', '')
@@ -310,8 +384,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_process_cmdline(self):
-        if self.pid == 0:
-            return []   # special case for kernel process
         f = open("/proc/%s/cmdline" % self.pid)
         try:
             # return the args as a list
@@ -321,8 +393,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_process_terminal(self):
-        if self.pid == 0:
-            return None   # special case for kernel process
         f = open("/proc/%s/stat" % self.pid)
         try:
             tty_nr = int(f.read().split(' ')[6])
@@ -335,9 +405,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_process_io_counters(self):
-        # special case for 0 (kernel process) PID
-        if self.pid == 0:
-            return ntuple_io(0, 0, 0, 0)
         f = open("/proc/%s/io" % self.pid)
         try:
             for line in f:
@@ -355,9 +422,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_cpu_times(self):
-        # special case for 0 (kernel process) PID
-        if self.pid == 0:
-            return ntuple_cputimes(0.0, 0.0)
         f = open("/proc/%s/stat" % self.pid)
         try:
             st = f.read().strip()
@@ -379,9 +443,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_process_create_time(self):
-        # special case for 0 (kernel processes) PID; return system uptime
-        if self.pid == 0:
-            return BOOT_TIME
         f = open("/proc/%s/stat" % self.pid)
         try:
             st = f.read().strip()
@@ -399,9 +460,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_memory_info(self):
-        # special case for 0 (kernel processes) PID
-        if self.pid == 0:
-            return ntuple_meminfo(0, 0)
         f = open("/proc/%s/status" % self.pid)
         try:
             virtual_size = 0
@@ -420,8 +478,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_process_cwd(self):
-        if self.pid == 0:
-            raise AccessDenied(self.pid, self._process_name)
         # readlink() might return paths containing null bytes causing
         # problems when used with other fs-related functions (os.*,
         # open(), ...)
@@ -430,8 +486,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_process_num_threads(self):
-        if self.pid == 0:
-            return 0
         f = open("/proc/%s/status" % self.pid)
         try:
             for line in f:
@@ -443,8 +497,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_process_threads(self):
-        if self.pid == 0:
-            return []
         thread_ids = os.listdir("/proc/%s/task" % self.pid)
         thread_ids.sort()
         retlist = []
@@ -516,8 +568,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_process_status(self):
-        if self.pid == 0:
-            return 0
         f = open("/proc/%s/status" % self.pid)
         try:
             for line in f:
@@ -531,8 +581,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_open_files(self):
-        if self.pid == 0:
-            return []
         retlist = []
         files = os.listdir("/proc/%s/fd" % self.pid)
         for fd in files:
@@ -551,9 +599,23 @@ class Process(object):
         return retlist
 
     @wrap_exceptions
-    def get_connections(self):
-        if self.pid == 0:
-            return []
+    def get_connections(self, kind='inet'):
+        """Return connections opened by process as a list of namedtuples.
+        The kind parameter filters for connections that fit the following
+        criteria:
+
+        Kind Value      Number of connections using
+        inet            IPv4 and IPv6
+        inet4           IPv4
+        inet6           IPv6
+        tcp             TCP
+        tcp4            TCP over IPv4
+        tcp6            TCP over IPv6
+        udp             UDP
+        udp4            UDP over IPv4
+        udp6            UDP over IPv6
+        all             the sum of all the possible families and protocols
+        """
         inodes = {}
         # os.listdir() is gonna raise a lot of access denied
         # exceptions in case of unprivileged user; that's fine:
@@ -572,34 +634,65 @@ class Process(object):
             # no connections for this process
             return []
 
-        def process(file, family, _type):
+        def process(file, family, type_):
             retlist = []
-            f = open(file)
+            try:
+                f = open(file, 'r')
+            except IOError, err:
+                # IPv6 not supported on this platform
+                if err.errno == errno.ENOENT and file.endswith('6'):
+                    return []
+                else:
+                    raise
             try:
                 f.readline()  # skip the first line
                 for line in f:
-                    _, laddr, raddr, status, _, _, _, _, _, inode = \
-                                                            line.split()[:10]
-                    if inode in inodes:
-                        laddr = self._decode_address(laddr, family)
-                        raddr = self._decode_address(raddr, family)
-                        if _type == socket.SOCK_STREAM:
-                            status = _TCP_STATES_TABLE[status]
-                        else:
-                            status = ""
-                        fd = int(inodes[inode])
-                        conn = ntuple_connection(fd, family, _type, laddr,
-                                                 raddr, status)
-                        retlist.append(conn)
+                    # IPv4 / IPv6
+                    if family in (socket.AF_INET, socket.AF_INET6):
+                        _, laddr, raddr, status, _, _, _, _, _, inode = \
+                                                                line.split()[:10]
+                        if inode in inodes:
+                            laddr = self._decode_address(laddr, family)
+                            raddr = self._decode_address(raddr, family)
+                            if type_ == socket.SOCK_STREAM:
+                                status = _TCP_STATES_TABLE[status]
+                            else:
+                                status = ""
+                            fd = int(inodes[inode])
+                            conn = ntuple_connection(fd, family, type_, laddr,
+                                                     raddr, status)
+                            retlist.append(conn)
+                    else:
+                        raise ValueError(family)
                 return retlist
             finally:
                 f.close()
 
-        tcp4 = process("/proc/net/tcp", socket.AF_INET, socket.SOCK_STREAM)
-        tcp6 = process("/proc/net/tcp6", socket.AF_INET6, socket.SOCK_STREAM)
-        udp4 = process("/proc/net/udp", socket.AF_INET, socket.SOCK_DGRAM)
-        udp6 = process("/proc/net/udp6", socket.AF_INET6, socket.SOCK_DGRAM)
-        return tcp4 + tcp6 + udp4 + udp6
+        tcp4 = ("tcp" , socket.AF_INET , socket.SOCK_STREAM)
+        tcp6 = ("tcp6", socket.AF_INET6, socket.SOCK_STREAM)
+        udp4 = ("udp" , socket.AF_INET , socket.SOCK_DGRAM)
+        udp6 = ("udp6", socket.AF_INET6, socket.SOCK_DGRAM)
+
+        tmap = {
+            "all"  : (tcp4, tcp6, udp4, udp6),
+            "tcp"  : (tcp4, tcp6),
+            "tcp4" : (tcp4,),
+            "tcp6" : (tcp6,),
+            "udp"  : (udp4, udp6),
+            "udp4" : (udp4,),
+            "udp6" : (udp6,),
+            "inet" : (tcp4, tcp6, udp4, udp6),
+            "inet4": (tcp4, udp4),
+            "inet6": (tcp6, udp6),
+        }
+        if kind not in tmap:
+            raise ValueError("invalid %r kind argument; choose between %s"
+                             % (kind, ', '.join([repr(x) for x in tmap])))
+        ret = []
+        for f, family, type_ in tmap[kind]:
+            ret += process("/proc/net/%s" % f, family, type_)
+        return ret
+
 
 #    --- lsof implementation
 #
@@ -609,8 +702,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_process_ppid(self):
-        if self.pid == 0:
-            return 0
         f = open("/proc/%s/status" % self.pid)
         try:
             for line in f:
@@ -623,8 +714,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_process_uids(self):
-        if self.pid == 0:
-            return ntuple_uids(0, 0, 0)
         f = open("/proc/%s/status" % self.pid)
         try:
             for line in f:
@@ -637,8 +726,6 @@ class Process(object):
 
     @wrap_exceptions
     def get_process_gids(self):
-        if self.pid == 0:
-            return ntuple_uids(0, 0, 0)
         f = open("/proc/%s/status" % self.pid)
         try:
             for line in f:
@@ -657,9 +744,9 @@ class Process(object):
         "0500000A:0016" -> ("10.0.0.5", 22)
         "0000000000000000FFFF00000100007F:9E49" -> ("::ffff:127.0.0.1", 40521)
 
-        The IPv4 address portion is a little-endian four-byte hexadecimal
-        number; that is, the least significant byte is listed first,
-        so we need to reverse the order of the bytes to convert it
+        The IP address portion is a little or big endian four-byte
+        hexadecimal number; that is, the least significant byte is listed
+        first, so we need to reverse the order of the bytes to convert it
         to an IP address.
         The port is represented as a two-byte hexadecimal number.
 
@@ -675,13 +762,22 @@ class Process(object):
         if not port:
             return ()
         if family == socket.AF_INET:
-            ip = socket.inet_ntop(family, base64.b16decode(ip)[::-1])
+            # see: http://code.google.com/p/psutil/issues/detail?id=201
+            if sys.byteorder == 'little':
+                ip = socket.inet_ntop(family, base64.b16decode(ip)[::-1])
+            else:
+                ip = socket.inet_ntop(family, base64.b16decode(ip))
         else:  # IPv6
             # old version - let's keep it, just in case...
             #ip = ip.decode('hex')
             #return socket.inet_ntop(socket.AF_INET6,
             #          ''.join(ip[i:i+4][::-1] for i in xrange(0, 16, 4)))
             ip = base64.b16decode(ip)
-            ip = socket.inet_ntop(socket.AF_INET6,
+            # see: http://code.google.com/p/psutil/issues/detail?id=201
+            if sys.byteorder == 'little':
+                ip = socket.inet_ntop(socket.AF_INET6,
                                 struct.pack('>4I', *struct.unpack('<4I', ip)))
+            else:
+                ip = socket.inet_ntop(socket.AF_INET6,
+                                struct.pack('<4I', *struct.unpack('<4I', ip)))
         return (ip, port)
