@@ -2,6 +2,11 @@
 #
 # $Id$
 #
+# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+# Use of this source code is governed by a BSD-style license that can be
+# found in the LICENSE file.
+
+"""Windows platform implementation."""
 
 import errno
 import os
@@ -34,7 +39,8 @@ from _psutil_mswindows import (ABOVE_NORMAL_PRIORITY_CLASS,
                                HIGH_PRIORITY_CLASS,
                                IDLE_PRIORITY_CLASS,
                                NORMAL_PRIORITY_CLASS,
-                               REALTIME_PRIORITY_CLASS)
+                               REALTIME_PRIORITY_CLASS,
+                               INFINITE)
 
 # --- public functions
 
@@ -109,6 +115,8 @@ def get_system_per_cpu_times():
 
 get_pid_list = _psutil_mswindows.get_pid_list
 pid_exists = _psutil_mswindows.pid_exists
+network_io_counters = _psutil_mswindows.get_network_io_counters
+disk_io_counters = _psutil_mswindows.get_disk_io_counters
 
 # --- decorator
 
@@ -179,8 +187,8 @@ class Process(object):
 
     @wrap_exceptions
     def process_wait(self, timeout=None):
-        if not timeout:
-            timeout = 0
+        if timeout is None:
+            timeout = INFINITE
         else:
             # WaitForSingleObject() expects time in milliseconds
             timeout = int(timeout * 1000)
@@ -261,9 +269,13 @@ class Process(object):
         return retlist
 
     @wrap_exceptions
-    def get_connections(self):
-        retlist = _psutil_mswindows.get_process_connections(self.pid)
-        return [ntuple_connection(*conn) for conn in retlist]
+    def get_connections(self, kind='inet'):
+        if kind not in conn_tmap:
+            raise ValueError("invalid %r kind argument; choose between %s"
+                             % (kind, ', '.join([repr(x) for x in conn_tmap])))
+        families, types = conn_tmap[kind]
+        ret = _psutil_mswindows.get_process_connections(self.pid, families, types)
+        return [ntuple_connection(*conn) for conn in ret]
 
     @wrap_exceptions
     def get_process_nice(self):
