@@ -2,6 +2,9 @@
 #
 # $Id$
 #
+# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+# Use of this source code is governed by a BSD-style license that can be
+# found in the LICENSE file.
 
 """Routines common to all posix systems."""
 
@@ -44,12 +47,12 @@ def wait_pid(pid, timeout=None):
     Raise TimeoutExpired on timeout expired.
     """
     def check_timeout():
-        if timeout:
+        if timeout is not None:
             if time.time() >= stop_at:
                 raise TimeoutExpired
             time.sleep(0.001)
 
-    if timeout:
+    if timeout is not None:
         waitcall = lambda: os.waitpid(pid, os.WNOHANG)
         stop_at = time.time() + timeout
     else:
@@ -99,7 +102,7 @@ def get_disk_usage(path):
     # reserved blocks that we are currently not considering:
     # http://goo.gl/sWGbH
     return ntuple_diskinfo(total, used, free, percent)
-
+    
 def _get_terminal_map():
     ret = {}
     ls = glob.glob('/dev/tty*') + glob.glob('/dev/pts/*')
@@ -125,6 +128,7 @@ class LsofParser:
         self.pid = pid
         self.process_name = name
 
+    # XXX - this is no longer used
     def get_process_open_files(self):
         """Return files opened by process by parsing lsof output."""
         # Options:
