@@ -1,18 +1,30 @@
 /*
  * $Id$
  *
+ * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
+ *
  * Windows platform-specific module methods for _psutil_mswindows
  */
 
+// Fixes clash between winsock2.h and windows.h
+#define WIN32_LEAN_AND_MEAN
+
 #include <Python.h>
 #include <windows.h>
 #include <Psapi.h>
 #include <time.h>
 #include <lm.h>
+#include <WinIoCtl.h>
 #include <tchar.h>
 #include <tlhelp32.h>
+#include <winsock2.h>
 #include <iphlpapi.h>
 
+// Link with Iphlpapi.lib
+#pragma comment(lib, "IPHLPAPI.lib")
+
 #include "_psutil_mswindows.h"
 #include "_psutil_common.h"
 #include "arch/mswindows/security.h"
@@ -169,9 +181,6 @@ process_wait(PyObject* self, PyObject* args)
     if (pid == 0) {
         return AccessDenied();
     }
-    if (timeout == 0) {
-        timeout = INFINITE;
-    }
 
     hProcess = OpenProcess(SYNCHRONIZE | PROCESS_QUERY_INFORMATION, FALSE, pid);
     if (hProcess == NULL) {
@@ -474,7 +483,15 @@ get_memory_info(PyObject* self, PyObject* args)
     }
 
     CloseHandle(hProcess);
+
+// py 2.4
+#if (PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION <= 4)
+    return Py_BuildValue("(II)", (unsigned int)counters.WorkingSetSize,
+                                 (unsigned int)counters.PagefileUsage);
+#else
+// py >= 2.5
     return Py_BuildValue("(nn)", counters.WorkingSetSize, counters.PagefileUsage);
+#endif
 }
 
 
@@ -772,6 +789,7 @@ suspend_resume_process(DWORD pid, int suspend)
         }
     } while (Thread32Next(hThreadSnap, &te32));
 
+    CloseHandle(hThreadSnap);
     return TRUE;
 }
 
@@ -1203,6 +1221,11 @@ typedef struct _MIB_UDP6TABLE_OWNER_PID
 } MIB_UDP6TABLE_OWNER_PID, *PMIB_UDP6TABLE_OWNER_PID;
 
 
+#define ConnDecrefPyObjs() Py_DECREF(_AF_INET); \
+                           Py_DECREF(_AF_INET6);\
+                           Py_DECREF(_SOCK_STREAM);\
+                           Py_DECREF(_SOCK_DGRAM);
+
 /*
  * Return a list of network connections opened by a process
  */
@@ -1214,6 +1237,14 @@ get_process_connections(PyObject* self, PyObject* args)
     unsigned long pid;
     PyObject* connectionsList;
     PyObject* connectionTuple;
+    PyObject *af_filter = NULL;
+    PyObject *type_filter = NULL;
+
+    PyObject *_AF_INET = PyLong_FromLong((long)AF_INET);
+    PyObject *_AF_INET6 = PyLong_FromLong((long)AF_INET6);
+    PyObject *_SOCK_STREAM = PyLong_FromLong((long)SOCK_STREAM);
+    PyObject *_SOCK_DGRAM = PyLong_FromLong((long)SOCK_DGRAM);
+
     typedef PSTR (NTAPI *_RtlIpv4AddressToStringA)(struct in_addr *,
                                                    PSTR /* __out_ecount(16) */);
     _RtlIpv4AddressToStringA rtlIpv4AddressToStringA;
@@ -1238,11 +1269,19 @@ get_process_connections(PyObject* self, PyObject* args)
     CHAR addressBufferRemote[65];
     PyObject* addressTupleRemote;
 
-    if (!PyArg_ParseTuple(args, "l", &pid)) {
+    if (! PyArg_ParseTuple(args, "lOO", &pid, &af_filter, &type_filter)) {
+        ConnDecrefPyObjs();
+        return NULL;
+    }
+
+    if (!PySequence_Check(af_filter) || !PySequence_Check(type_filter)) {
+        ConnDecrefPyObjs();
+        PyErr_SetString(PyExc_TypeError, "arg 2 or 3 is not a sequence");
         return NULL;
     }
 
     if (pid_is_running(pid) == 0) {
+        ConnDecrefPyObjs();
         return NoSuchProcess();
     }
 
@@ -1268,6 +1307,7 @@ get_process_connections(PyObject* self, PyObject* args)
     if ((getExtendedTcpTable == NULL) || (getExtendedUdpTable == NULL)) {
         PyErr_SetString(PyExc_NotImplementedError,
                         "feature not supported on this Windows version");
+        ConnDecrefPyObjs();
         return NULL;
     }
 
@@ -1275,230 +1315,253 @@ get_process_connections(PyObject* self, PyObject* args)
 
     /* TCP IPv4 */
 
-    tableSize = 0;
-    getExtendedTcpTable(NULL, &tableSize, FALSE, AF_INET,
-                        TCP_TABLE_OWNER_PID_ALL, 0);
-
-    table = malloc(tableSize);
-
-    if (getExtendedTcpTable(table, &tableSize, FALSE, AF_INET,
-                            TCP_TABLE_OWNER_PID_ALL, 0) == 0)
+    if ((PySequence_Contains(af_filter, _AF_INET) == 1) &&
+        (PySequence_Contains(type_filter, _SOCK_STREAM) == 1))
     {
-        tcp4Table = table;
+        tableSize = 0;
+        getExtendedTcpTable(NULL, &tableSize, FALSE, AF_INET,
+                            TCP_TABLE_OWNER_PID_ALL, 0);
 
-        for (i = 0; i < tcp4Table->dwNumEntries; i++)
+        table = malloc(tableSize);
+
+        if (getExtendedTcpTable(table, &tableSize, FALSE, AF_INET,
+                                TCP_TABLE_OWNER_PID_ALL, 0) == 0)
         {
-            if (tcp4Table->table[i].dwOwningPid != pid) {
-                continue;
-            }
+            tcp4Table = table;
 
-            if (tcp4Table->table[i].dwLocalAddr != 0 ||
-                tcp4Table->table[i].dwLocalPort != 0)
+            for (i = 0; i < tcp4Table->dwNumEntries; i++)
             {
-                struct in_addr addr;
+                if (tcp4Table->table[i].dwOwningPid != pid) {
+                    continue;
+                }
 
-                addr.S_un.S_addr = tcp4Table->table[i].dwLocalAddr;
-                rtlIpv4AddressToStringA(&addr, addressBufferLocal);
-                addressTupleLocal = Py_BuildValue("(si)", addressBufferLocal,
-                           BYTESWAP_USHORT(tcp4Table->table[i].dwLocalPort));
-            }
-            else
-            {
-                addressTupleLocal = PyTuple_New(0);
-            }
+                if (tcp4Table->table[i].dwLocalAddr != 0 ||
+                    tcp4Table->table[i].dwLocalPort != 0)
+                {
+                    struct in_addr addr;
 
-            // On Windows <= XP, remote addr is filled even if socket
-            // is in LISTEN mode in which case we just ignore it.
-            if ((tcp4Table->table[i].dwRemoteAddr != 0 ||
-                 tcp4Table->table[i].dwRemotePort != 0) &&
-                (tcp4Table->table[i].dwState != MIB_TCP_STATE_LISTEN))
-            {
-                struct in_addr addr;
+                    addr.S_un.S_addr = tcp4Table->table[i].dwLocalAddr;
+                    rtlIpv4AddressToStringA(&addr, addressBufferLocal);
+                    addressTupleLocal = Py_BuildValue("(si)", addressBufferLocal,
+                               BYTESWAP_USHORT(tcp4Table->table[i].dwLocalPort));
+                }
+                else
+                {
+                    addressTupleLocal = PyTuple_New(0);
+                }
 
-                addr.S_un.S_addr = tcp4Table->table[i].dwRemoteAddr;
-                rtlIpv4AddressToStringA(&addr, addressBufferRemote);
-                addressTupleRemote = Py_BuildValue("(si)", addressBufferRemote,
-                        BYTESWAP_USHORT(tcp4Table->table[i].dwRemotePort));
-            }
-            else
-            {
-                addressTupleRemote = PyTuple_New(0);
-            }
+                // On Windows <= XP, remote addr is filled even if socket
+                // is in LISTEN mode in which case we just ignore it.
+                if ((tcp4Table->table[i].dwRemoteAddr != 0 ||
+                     tcp4Table->table[i].dwRemotePort != 0) &&
+                    (tcp4Table->table[i].dwState != MIB_TCP_STATE_LISTEN))
+                {
+                    struct in_addr addr;
 
-            connectionTuple = Py_BuildValue("(iiiNNs)",
-                -1,
-                AF_INET,
-                SOCK_STREAM,
-                addressTupleLocal,
-                addressTupleRemote,
-                state_to_string(tcp4Table->table[i].dwState)
-                );
-            PyList_Append(connectionsList, connectionTuple);
+                    addr.S_un.S_addr = tcp4Table->table[i].dwRemoteAddr;
+                    rtlIpv4AddressToStringA(&addr, addressBufferRemote);
+                    addressTupleRemote = Py_BuildValue("(si)", addressBufferRemote,
+                            BYTESWAP_USHORT(tcp4Table->table[i].dwRemotePort));
+                }
+                else
+                {
+                    addressTupleRemote = PyTuple_New(0);
+                }
+
+                connectionTuple = Py_BuildValue("(iiiNNs)",
+                    -1,
+                    AF_INET,
+                    SOCK_STREAM,
+                    addressTupleLocal,
+                    addressTupleRemote,
+                    state_to_string(tcp4Table->table[i].dwState)
+                    );
+                PyList_Append(connectionsList, connectionTuple);
+                Py_DECREF(connectionTuple);
+            }
         }
-    }
 
-    free(table);
+        free(table);
+    }
 
     /* TCP IPv6 */
 
-    tableSize = 0;
-    getExtendedTcpTable(NULL, &tableSize, FALSE, AF_INET6,
-                        TCP_TABLE_OWNER_PID_ALL, 0);
-
-    table = malloc(tableSize);
-
-    if (getExtendedTcpTable(table, &tableSize, FALSE, AF_INET6,
-                            TCP_TABLE_OWNER_PID_ALL, 0) == 0)
+    if ((PySequence_Contains(af_filter, _AF_INET6) == 1) &&
+        (PySequence_Contains(type_filter, _SOCK_STREAM) == 1))
     {
-        tcp6Table = table;
+        tableSize = 0;
+        getExtendedTcpTable(NULL, &tableSize, FALSE, AF_INET6,
+                            TCP_TABLE_OWNER_PID_ALL, 0);
 
-        for (i = 0; i < tcp6Table->dwNumEntries; i++)
+        table = malloc(tableSize);
+
+        if (getExtendedTcpTable(table, &tableSize, FALSE, AF_INET6,
+                                 TCP_TABLE_OWNER_PID_ALL, 0) == 0)
         {
-            if (tcp6Table->table[i].dwOwningPid != pid) {
-                continue;
-            }
+            tcp6Table = table;
 
-            if (memcmp(tcp6Table->table[i].ucLocalAddr, null_address, 16) != 0 ||
-            tcp6Table->table[i].dwLocalPort != 0)
+            for (i = 0; i < tcp6Table->dwNumEntries; i++)
             {
-                struct in6_addr addr;
+                if (tcp6Table->table[i].dwOwningPid != pid) {
+                    continue;
+                }
 
-                memcpy(&addr, tcp6Table->table[i].ucLocalAddr, 16);
-                rtlIpv6AddressToStringA(&addr, addressBufferLocal);
-                addressTupleLocal = Py_BuildValue("(si)", addressBufferLocal,
-                         BYTESWAP_USHORT(tcp6Table->table[i].dwLocalPort));
-            }
-            else
-            {
-                addressTupleLocal = PyTuple_New(0);
-            }
+                if (memcmp(tcp6Table->table[i].ucLocalAddr, null_address, 16) != 0 ||
+                tcp6Table->table[i].dwLocalPort != 0)
+                {
+                    struct in6_addr addr;
 
-            // On Windows <= XP, remote addr is filled even if socket
-            // is in LISTEN mode in which case we just ignore it.
-            if ((memcmp(tcp6Table->table[i].ucRemoteAddr, null_address, 16) != 0 ||
-                tcp6Table->table[i].dwRemotePort != 0) &&
-               (tcp6Table->table[i].dwState != MIB_TCP_STATE_LISTEN))
-            {
-                struct in6_addr addr;
+                    memcpy(&addr, tcp6Table->table[i].ucLocalAddr, 16);
+                    rtlIpv6AddressToStringA(&addr, addressBufferLocal);
+                    addressTupleLocal = Py_BuildValue("(si)", addressBufferLocal,
+                             BYTESWAP_USHORT(tcp6Table->table[i].dwLocalPort));
+                }
+                else
+                {
+                    addressTupleLocal = PyTuple_New(0);
+                }
 
-                memcpy(&addr, tcp6Table->table[i].ucRemoteAddr, 16);
-                rtlIpv6AddressToStringA(&addr, addressBufferRemote);
-                addressTupleRemote = Py_BuildValue("(si)", addressBufferRemote,
-                        BYTESWAP_USHORT(tcp6Table->table[i].dwRemotePort));
-            }
-            else
-            {
-                addressTupleRemote = PyTuple_New(0);
-            }
+                // On Windows <= XP, remote addr is filled even if socket
+                // is in LISTEN mode in which case we just ignore it.
+                if ((memcmp(tcp6Table->table[i].ucRemoteAddr, null_address, 16) != 0 ||
+                    tcp6Table->table[i].dwRemotePort != 0) &&
+                   (tcp6Table->table[i].dwState != MIB_TCP_STATE_LISTEN))
+                {
+                    struct in6_addr addr;
 
-            connectionTuple = Py_BuildValue("(iiiNNs)",
-                -1,
-                AF_INET6,
-                SOCK_STREAM,
-                addressTupleLocal,
-                addressTupleRemote,
-                state_to_string(tcp6Table->table[i].dwState)
-                );
-            PyList_Append(connectionsList, connectionTuple);
+                    memcpy(&addr, tcp6Table->table[i].ucRemoteAddr, 16);
+                    rtlIpv6AddressToStringA(&addr, addressBufferRemote);
+                    addressTupleRemote = Py_BuildValue("(si)", addressBufferRemote,
+                            BYTESWAP_USHORT(tcp6Table->table[i].dwRemotePort));
+                }
+                else
+                {
+                    addressTupleRemote = PyTuple_New(0);
+                }
+
+                connectionTuple = Py_BuildValue("(iiiNNs)",
+                    -1,
+                    AF_INET6,
+                    SOCK_STREAM,
+                    addressTupleLocal,
+                    addressTupleRemote,
+                    state_to_string(tcp6Table->table[i].dwState)
+                    );
+                PyList_Append(connectionsList, connectionTuple);
+                Py_DECREF(connectionTuple);
+            }
         }
-    }
 
-    free(table);
+        free(table);
+    }
 
     /* UDP IPv4 */
 
-    tableSize = 0;
-    getExtendedUdpTable(NULL, &tableSize, FALSE, AF_INET,
-                        UDP_TABLE_OWNER_PID, 0);
-
-    table = malloc(tableSize);
-
-    if (getExtendedUdpTable(table, &tableSize, FALSE, AF_INET,
-                            UDP_TABLE_OWNER_PID, 0) == 0)
+    if ((PySequence_Contains(af_filter, _AF_INET) == 1) &&
+        (PySequence_Contains(type_filter, _SOCK_DGRAM) == 1))
     {
-        udp4Table = table;
+        tableSize = 0;
+        getExtendedUdpTable(NULL, &tableSize, FALSE, AF_INET,
+                            UDP_TABLE_OWNER_PID, 0);
 
-        for (i = 0; i < udp4Table->dwNumEntries; i++)
+        table = malloc(tableSize);
+
+        if (getExtendedUdpTable(table, &tableSize, FALSE, AF_INET,
+                                 UDP_TABLE_OWNER_PID, 0) == 0)
         {
-            if (udp4Table->table[i].dwOwningPid != pid) {
-                continue;
-            }
+            udp4Table = table;
 
-            if (udp4Table->table[i].dwLocalAddr != 0 ||
-                udp4Table->table[i].dwLocalPort != 0)
+            for (i = 0; i < udp4Table->dwNumEntries; i++)
             {
-                struct in_addr addr;
+                if (udp4Table->table[i].dwOwningPid != pid) {
+                    continue;
+                }
 
-                addr.S_un.S_addr = udp4Table->table[i].dwLocalAddr;
-                rtlIpv4AddressToStringA(&addr, addressBufferLocal);
-                addressTupleLocal = Py_BuildValue("(si)", addressBufferLocal,
-                            BYTESWAP_USHORT(udp4Table->table[i].dwLocalPort));
-            }
-            else
-            {
-                addressTupleLocal = PyTuple_New(0);
-            }
+                if (udp4Table->table[i].dwLocalAddr != 0 ||
+                    udp4Table->table[i].dwLocalPort != 0)
+                {
+                    struct in_addr addr;
 
-            connectionTuple = Py_BuildValue("(iiiNNs)",
-                -1,
-                AF_INET,
-                SOCK_DGRAM,
-                addressTupleLocal,
-                PyTuple_New(0),
-                ""
-                );
-            PyList_Append(connectionsList, connectionTuple);
+                    addr.S_un.S_addr = udp4Table->table[i].dwLocalAddr;
+                    rtlIpv4AddressToStringA(&addr, addressBufferLocal);
+                    addressTupleLocal = Py_BuildValue("(si)", addressBufferLocal,
+                                BYTESWAP_USHORT(udp4Table->table[i].dwLocalPort));
+                }
+                else
+                {
+                    addressTupleLocal = PyTuple_New(0);
+                }
+
+                connectionTuple = Py_BuildValue("(iiiNNs)",
+                    -1,
+                    AF_INET,
+                    SOCK_DGRAM,
+                    addressTupleLocal,
+                    PyTuple_New(0),
+                    ""
+                    );
+                PyList_Append(connectionsList, connectionTuple);
+                Py_DECREF(connectionTuple);
+            }
         }
-    }
 
-    free(table);
+        free(table);
+    }
 
     /* UDP IPv6 */
 
-    tableSize = 0;
-    getExtendedUdpTable(NULL, &tableSize, FALSE, AF_INET6, UDP_TABLE_OWNER_PID, 0);
-
-    table = malloc(tableSize);
-
-    if (getExtendedUdpTable(table, &tableSize, FALSE, AF_INET6, UDP_TABLE_OWNER_PID, 0) == 0)
+    if ((PySequence_Contains(af_filter, _AF_INET6) == 1) &&
+        (PySequence_Contains(type_filter, _SOCK_DGRAM) == 1))
     {
-        udp6Table = table;
+        tableSize = 0;
+        getExtendedUdpTable(NULL, &tableSize, FALSE,
+                            AF_INET6, UDP_TABLE_OWNER_PID, 0);
 
-        for (i = 0; i < udp6Table->dwNumEntries; i++)
+        table = malloc(tableSize);
+
+        if (getExtendedUdpTable(table, &tableSize, FALSE, AF_INET6,
+                                 UDP_TABLE_OWNER_PID, 0) == 0)
         {
-            if (udp6Table->table[i].dwOwningPid != pid) {
-                continue;
-            }
+            udp6Table = table;
 
-            if (memcmp(udp6Table->table[i].ucLocalAddr, null_address, 16) != 0 ||
-                udp6Table->table[i].dwLocalPort != 0)
+            for (i = 0; i < udp6Table->dwNumEntries; i++)
             {
-                struct in6_addr addr;
+                if (udp6Table->table[i].dwOwningPid != pid) {
+                    continue;
+                }
 
-                memcpy(&addr, udp6Table->table[i].ucLocalAddr, 16);
-                rtlIpv6AddressToStringA(&addr, addressBufferLocal);
-                addressTupleLocal = Py_BuildValue("(si)", addressBufferLocal,
-                          BYTESWAP_USHORT(udp6Table->table[i].dwLocalPort));
-            }
-            else
-            {
-                addressTupleLocal = PyTuple_New(0);
-            }
+                if (memcmp(udp6Table->table[i].ucLocalAddr, null_address, 16) != 0 ||
+                    udp6Table->table[i].dwLocalPort != 0)
+                {
+                    struct in6_addr addr;
 
-            connectionTuple = Py_BuildValue("(iiiNNs)",
-                -1,
-                AF_INET6,
-                SOCK_DGRAM,
-                addressTupleLocal,
-                PyTuple_New(0),
-                ""
-                );
-            PyList_Append(connectionsList, connectionTuple);
+                    memcpy(&addr, udp6Table->table[i].ucLocalAddr, 16);
+                    rtlIpv6AddressToStringA(&addr, addressBufferLocal);
+                    addressTupleLocal = Py_BuildValue("(si)", addressBufferLocal,
+                              BYTESWAP_USHORT(udp6Table->table[i].dwLocalPort));
+                }
+                else
+                {
+                    addressTupleLocal = PyTuple_New(0);
+                }
+
+                connectionTuple = Py_BuildValue("(iiiNNs)",
+                    -1,
+                    AF_INET6,
+                    SOCK_DGRAM,
+                    addressTupleLocal,
+                    PyTuple_New(0),
+                    ""
+                    );
+                PyList_Append(connectionsList, connectionTuple);
+                Py_DECREF(connectionTuple);
+            }
         }
+
+        free(table);
     }
 
-    free(table);
-
+    ConnDecrefPyObjs();
     return connectionsList;
 }
 
@@ -1657,7 +1720,158 @@ get_disk_usage(PyObject* self, PyObject* args)
 
 
 /*
- * Return disk partitions as a list of namedtuples.
+ * Return a Python list of named tuples with overall network I/O information
+ */
+static PyObject*
+get_network_io_counters(PyObject* self, PyObject* args)
+{
+    PyObject* py_retdict = PyDict_New();
+    PyObject* py_nic_info = NULL;
+    PyObject* py_pre_nic_name = NULL;
+    PyObject* py_nic_name = NULL;
+
+    int attempts = 0;
+    int outBufLen = 15000;
+    DWORD dwRetVal = 0;
+    MIB_IFROW *pIfRow;
+    ULONG flags = 0;
+    ULONG family = AF_UNSPEC;
+    PIP_ADAPTER_ADDRESSES pAddresses = NULL;
+    PIP_ADAPTER_ADDRESSES pCurrAddresses = NULL;
+
+    do {
+        pAddresses = (IP_ADAPTER_ADDRESSES *) malloc(outBufLen);
+        if (pAddresses == NULL) {
+            Py_DECREF(py_retdict);
+            PyErr_SetString(PyExc_RuntimeError,
+                "memory allocation failed for IP_ADAPTER_ADDRESSES struct.");
+            return NULL;
+        }
+
+        dwRetVal = GetAdaptersAddresses(family, flags, NULL, pAddresses,
+                                        &outBufLen);
+        if (dwRetVal == ERROR_BUFFER_OVERFLOW) {
+            free(pAddresses);
+            pAddresses = NULL;
+        }
+        else {
+            break;
+        }
+
+        attempts++;
+    } while ((dwRetVal == ERROR_BUFFER_OVERFLOW) && (attempts < 3));
+
+    if (dwRetVal != NO_ERROR) {
+        Py_DECREF(py_retdict);
+        PyErr_SetString(PyExc_RuntimeError,  "GetAdaptersAddresses() failed.");
+        return NULL;
+    }
+
+    pCurrAddresses = pAddresses;
+    while (pCurrAddresses) {
+        pIfRow = (MIB_IFROW *) malloc(sizeof(MIB_IFROW));
+
+        if (pIfRow == NULL) {
+            Py_DECREF(py_retdict);
+            Py_XDECREF(py_pre_nic_name);
+            Py_XDECREF(py_nic_name);
+            Py_XDECREF(py_nic_info);
+            PyErr_SetString(PyExc_RuntimeError,
+                "memory allocation failed for MIB_IFROW struct.");
+            return NULL;
+        }
+
+        pIfRow->dwIndex = pCurrAddresses->IfIndex;
+        dwRetVal = GetIfEntry(pIfRow);
+        if (dwRetVal != NO_ERROR) {
+            Py_DECREF(py_retdict);
+            Py_XDECREF(py_pre_nic_name);
+            Py_XDECREF(py_nic_name);
+            Py_XDECREF(py_nic_info);
+            PyErr_SetString(PyExc_RuntimeError,
+                "GetIfEntry() failed.");
+            return NULL;
+        }
+
+        py_nic_info = Py_BuildValue("(IIII)",
+                                    pIfRow->dwOutOctets,
+                                    pIfRow->dwInOctets,
+                                    pIfRow->dwOutUcastPkts,
+                                    pIfRow->dwInUcastPkts);
+
+        py_pre_nic_name = PyUnicode_FromWideChar(
+                                pCurrAddresses->FriendlyName,
+                                wcslen(pCurrAddresses->FriendlyName));
+        py_nic_name = PyUnicode_FromObject(py_pre_nic_name);
+        PyDict_SetItem(py_retdict, py_nic_name, py_nic_info);
+        Py_XDECREF(py_pre_nic_name);
+        Py_XDECREF(py_nic_name);
+        Py_XDECREF(py_nic_info);
+
+        free(pIfRow);
+        pCurrAddresses = pCurrAddresses->Next;
+    }
+
+    free(pAddresses);
+
+    return py_retdict;
+}
+
+
+/*
+ * Return a Python dict of tuples for disk I/O information
+ */
+static PyObject*
+get_disk_io_counters(PyObject* self, PyObject* args)
+{
+    PyObject* py_retdict = PyDict_New();
+    PyObject* py_disk_info;
+
+    DISK_PERFORMANCE diskPerformance;
+    DWORD dwSize;
+    HANDLE hDevice = NULL;
+    char szDevice[MAX_PATH];
+    char szDeviceDisplay[MAX_PATH];
+    int devNum;
+
+    for (devNum = 0;; devNum++) {
+        sprintf (szDevice, "\\\\.\\PhysicalDrive%d", devNum);
+        hDevice = CreateFile (szDevice, 0, FILE_SHARE_READ | FILE_SHARE_WRITE,
+                              NULL, OPEN_EXISTING, 0, NULL);
+
+        if (hDevice == INVALID_HANDLE_VALUE) {
+            // what happens if we get an invalid handle on the first disk?
+            // we might end up with an empty dict incorrectly in some cases
+            break;
+        }
+
+        if (DeviceIoControl (hDevice, IOCTL_DISK_PERFORMANCE, NULL, 0,
+            &diskPerformance, sizeof (DISK_PERFORMANCE), &dwSize, NULL)) {
+            sprintf (szDeviceDisplay, "PhysicalDrive%d", devNum);
+            py_disk_info = Py_BuildValue("(IILLLL)",
+                                         diskPerformance.ReadCount,
+                                         diskPerformance.WriteCount,
+                                         diskPerformance.BytesRead,
+                                         diskPerformance.BytesWritten,
+                                         (diskPerformance.ReadTime.QuadPart
+                                            * 10) / 1000,
+                                         (diskPerformance.WriteTime.QuadPart
+                                            * 10) / 1000);
+            PyDict_SetItemString(py_retdict,
+                                 szDeviceDisplay,
+                                 py_disk_info);
+            Py_XDECREF(py_disk_info);
+        }
+
+        CloseHandle(hDevice);
+    }
+
+    return py_retdict;
+}
+
+
+/*
+ * Return disk partitions as a list of strings.
  */
 static PyObject*
 win32_GetLogicalDriveStrings(PyObject* self, PyObject* args)
@@ -1666,6 +1880,7 @@ win32_GetLogicalDriveStrings(PyObject* self, PyObject* args)
     char drive_strings[255];
     char* drive_letter = drive_strings;
     PyObject* py_retlist = PyList_New(0);
+    PyObject* py_string = NULL;
 
     Py_BEGIN_ALLOW_THREADS
     num_bytes = GetLogicalDriveStrings(254, drive_letter);
@@ -1676,8 +1891,10 @@ win32_GetLogicalDriveStrings(PyObject* self, PyObject* args)
     }
 
     while (*drive_letter != 0) {
-        PyList_Append(py_retlist, Py_BuildValue("s", drive_letter));
-        drive_letter = strchr(drive_letter, 0) +1;
+        py_string = Py_BuildValue("s", drive_letter);
+        PyList_Append(py_retlist, py_string);
+        Py_DECREF(py_string);
+        drive_letter = strchr(drive_letter, 0) + 1;
     }
 
     return py_retlist;
@@ -1794,6 +2011,10 @@ PsutilMethods[] =
          "Return system per-cpu times as a list of tuples"},
      {"get_disk_usage", get_disk_usage, METH_VARARGS,
          "Return path's disk total and free as a Python tuple."},
+     {"get_network_io_counters", get_network_io_counters, METH_VARARGS,
+         "Return dict of tuples of networks I/O information."},
+     {"get_disk_io_counters", get_disk_io_counters, METH_VARARGS,
+         "Return dict of tuples of disks I/O information."},
 
      // --- windows API bindings
      {"win32_GetLogicalDriveStrings", win32_GetLogicalDriveStrings, METH_VARARGS,
@@ -1883,12 +2104,11 @@ struct module_state {
                                      NORMAL_PRIORITY_CLASS);
     PyModule_AddIntConstant(module, "REALTIME_PRIORITY_CLASS",
                                      REALTIME_PRIORITY_CLASS);
+    // private constants
+    PyModule_AddIntConstant(module, "INFINITE", INFINITE);
     SetSeDebug();
 
 #if PY_MAJOR_VERSION >= 3
     return module;
 #endif
 }
-
-
-
