@@ -1,8 +1,11 @@
 /*
  * $Id$
  *
- * Windows platform-specific module methods for _psutil_mswindows
+ * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  *
+ * Windows platform-specific module methods for _psutil_mswindows
  */
 
 #include <Python.h>
@@ -41,6 +44,8 @@ static PyObject* get_system_cpu_times(PyObject* self, PyObject* args);
 static PyObject* pid_exists(PyObject* self, PyObject* args);
 static PyObject* get_disk_usage(PyObject* self, PyObject* args);
 static PyObject* get_disk_partitions(PyObject* self, PyObject* args);
+static PyObject* get_network_io_counters(PyObject* self, PyObject* args);
+static PyObject* get_disk_io_counters(PyObject* self, PyObject* args);
 
 // --- windows API bindings
 
