@@ -1,6 +1,10 @@
 /*
  * $Id$
  *
+ * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
+ *
  * OS X platform-specific module methods for _psutil_osx
  */
 
@@ -16,6 +20,7 @@
 #include <sys/proc_info.h>
 #include <netinet/tcp_fsm.h>
 #include <arpa/inet.h>
+#include <net/if_dl.h>
 
 #include <mach/mach.h>
 #include <mach/task.h>
@@ -25,6 +30,12 @@
 #include <mach/mach_traps.h>
 #include <mach/shared_memory_server.h>
 
+#include <CoreFoundation/CoreFoundation.h>
+#include <IOKit/IOKitLib.h>
+#include <IOKit/storage/IOBlockStorageDriver.h>
+#include <IOKit/storage/IOMedia.h>
+#include <IOKit/IOBSD.h>
+
 #include "_psutil_osx.h"
 #include "_psutil_common.h"
 #include "arch/osx/process_info.h"
@@ -58,8 +69,8 @@ get_pid_list(PyObject* self, PyObject* args)
             Py_XDECREF(pid);
             proclist++;
         }
+        free(orig_address);
     }
-    free(orig_address);
     return retlist;
 }
 
@@ -735,7 +746,7 @@ get_process_threads(PyObject* self, PyObject* args)
                          (thread_info_t)thinfo, &thread_info_count);
         if (kr != KERN_SUCCESS) {
             return PyErr_Format(PyExc_RuntimeError, "thread_info() failed");
-        }        
+        }
         basic_info_th = (thread_basic_info_t)thinfo;
         // XXX - thread_info structure does not provide any process id;
         // the best we can do is assigning an incremental bogus value
@@ -843,13 +854,18 @@ get_process_open_files(PyObject* self, PyObject* args)
     return retList;
 
 error:
-    if (errno != 0)
+    if (errno != 0) {
         return PyErr_SetFromErrno(PyExc_OSError);
-    else if (! pid_exists(pid) )
+    }
+
+    else if (! pid_exists(pid) ) {
         return NoSuchProcess();
-    else
+    }
+
+    else {
         return PyErr_Format(PyExc_RuntimeError,
                             "proc_pidinfo(PROC_PIDLISTFDS) failed");
+    }
 }
 
 
@@ -907,13 +923,19 @@ get_process_connections(PyObject* self, PyObject* args)
     struct proc_fdinfo *fdp_pointer;
     struct socket_fdinfo si;
 
-
     PyObject *retList = PyList_New(0);
     PyObject *tuple = NULL;
     PyObject *laddr = NULL;
     PyObject *raddr = NULL;
+    PyObject *af_filter = NULL;
+    PyObject *type_filter = NULL;
 
-    if (! PyArg_ParseTuple(args, "l", &pid)) {
+    if (! PyArg_ParseTuple(args, "lOO", &pid, &af_filter, &type_filter)) {
+        return NULL;
+    }
+
+    if (!PySequence_Check(af_filter) || !PySequence_Check(type_filter)) {
+        PyErr_SetString(PyExc_TypeError, "arg 2 or 3 is not a sequence");
         return NULL;
     }
 
@@ -971,28 +993,45 @@ get_process_connections(PyObject* self, PyObject* args)
             int fd, family, type, lport, rport;
             char lip[200], rip[200];
             char *state;
+            int inseq;
+            PyObject* _family;
+            PyObject* _type;
 
             fd = (int)fdp_pointer->proc_fd;
             family = si.psi.soi_family;
             type = si.psi.soi_kind;
 
-            if ((family != AF_INET) && (family != AF_INET6)) {
+            if (type == 2) {
+                type = SOCK_STREAM;
+            }
+
+            else if (type == 1) {
+                type = SOCK_DGRAM;
+            }
+
+            else {
                 continue;
             }
 
-            if (type == 2)
-                type = SOCK_STREAM;
-            else if (type == 1)
-                type = SOCK_DGRAM;
-            else
+            // apply filters
+            _family = PyLong_FromLong((long)family);
+            inseq = PySequence_Contains(af_filter, _family);
+            Py_DECREF(_family);
+            if (inseq == 0) {
                 continue;
+            }
+
+            _type = PyLong_FromLong((long)type);
+            inseq = PySequence_Contains(type_filter, _type);
+            Py_DECREF(_type);
+            if (inseq == 0) {
+                continue;
+            }
 
             if (errno != 0) {
-                printf("errno 1 = %i\n", errno);
                 return PyErr_SetFromErrno(PyExc_OSError);
             }
 
-
             if (family == AF_INET) {
                 inet_ntop(AF_INET,
                           &si.psi.soi_proto.pri_tcp.tcpsi_ini.insi_laddr.ina_46.i46a_addr4,
@@ -1003,6 +1042,7 @@ get_process_connections(PyObject* self, PyObject* args)
                           rip,
                           sizeof(lip));
             }
+
             else {
                 inet_ntop(AF_INET6,
                           &si.psi.soi_proto.pri_tcp.tcpsi_ini.insi_laddr.ina_6,
@@ -1019,16 +1059,22 @@ get_process_connections(PyObject* self, PyObject* args)
 
             lport = ntohs(si.psi.soi_proto.pri_tcp.tcpsi_ini.insi_lport);
             rport = ntohs(si.psi.soi_proto.pri_tcp.tcpsi_ini.insi_fport);
-            if (type == SOCK_STREAM)
+            if (type == SOCK_STREAM) {
                 state = get_connection_status((int)si.psi.soi_proto.pri_tcp.tcpsi_state);
-            else
+            }
+
+            else {
                 state = "";
+            }
 
             laddr = Py_BuildValue("(si)", lip, lport);
-            if (rport != 0)
+            if (rport != 0) {
                 raddr = Py_BuildValue("(si)", rip, rport);
-            else
+            }
+
+            else {
                 raddr = PyTuple_New(0);
+            }
 
             // --- construct python list
             tuple = Py_BuildValue("(iiiNNs)", fd, family, type, laddr, raddr,
@@ -1042,13 +1088,244 @@ get_process_connections(PyObject* self, PyObject* args)
     return retList;
 
 error:
-    if (errno != 0)
+    if (errno != 0) {
         return PyErr_SetFromErrno(PyExc_OSError);
-    else if (! pid_exists(pid) )
+    }
+
+    else if (! pid_exists(pid) ) {
         return NoSuchProcess();
-    else
+    }
+
+    else {
         return PyErr_Format(PyExc_RuntimeError,
                             "proc_pidinfo(PROC_PIDLISTFDS) failed");
+    }
+}
+
+
+/*
+ * Return a Python list of named tuples with overall network I/O information
+ */
+static PyObject*
+get_network_io_counters(PyObject* self, PyObject* args)
+{
+    PyObject* py_retdict = PyDict_New();
+    PyObject* py_ifc_info;
+
+    char *buf = NULL, *lim, *next;
+    struct if_msghdr *ifm;
+    int mib[6];
+    size_t len;
+
+    mib[0] = CTL_NET;          // networking subsystem
+    mib[1] = PF_ROUTE;         // type of information
+    mib[2] = 0;                // protocol (IPPROTO_xxx)
+    mib[3] = 0;                // address family
+    mib[4] = NET_RT_IFLIST2;   // operation
+    mib[5] = 0;
+
+    if (sysctl(mib, 6, NULL, &len, NULL, 0) < 0) {
+        Py_DECREF(py_retdict);
+        PyErr_SetFromErrno(0);
+        return NULL;
+    }
+
+    buf = malloc(len);
+
+    if (sysctl(mib, 6, buf, &len, NULL, 0) < 0) {
+        if (buf) {
+            free(buf);
+        }
+        Py_DECREF(py_retdict);
+        PyErr_SetFromErrno(0);
+        return NULL;
+    }
+
+    lim = buf + len;
+
+    for (next = buf; next < lim; ) {
+        ifm = (struct if_msghdr *)next;
+        next += ifm->ifm_msglen;
+
+        if (ifm->ifm_type == RTM_IFINFO2) {
+            struct if_msghdr2 *if2m = (struct if_msghdr2 *)ifm;
+            struct sockaddr_dl *sdl = (struct sockaddr_dl *)(if2m + 1);
+            char ifc_name[32];
+
+            strncpy(ifc_name, sdl->sdl_data, sdl->sdl_nlen);
+            ifc_name[sdl->sdl_nlen] = 0;
+
+            py_ifc_info = Py_BuildValue("(KKKK)",
+                                        if2m->ifm_data.ifi_obytes,
+                                        if2m->ifm_data.ifi_ibytes,
+                                        if2m->ifm_data.ifi_opackets,
+                                        if2m->ifm_data.ifi_ipackets);
+            PyDict_SetItemString(py_retdict, ifc_name, py_ifc_info);
+            Py_XDECREF(py_ifc_info);
+        }
+        else {
+            continue;
+        }
+    }
+
+    free(buf);
+
+    return py_retdict;
+}
+
+
+/*
+ * Return a Python dict of tuples for disk I/O information
+ */
+static PyObject*
+get_disk_io_counters(PyObject* self, PyObject* args)
+{
+    PyObject* py_retdict = PyDict_New();
+    PyObject* py_disk_info;
+
+    CFDictionaryRef parent_dict;
+    CFDictionaryRef props_dict;
+    CFDictionaryRef stats_dict;
+    io_registry_entry_t parent;
+    io_registry_entry_t disk;
+    io_iterator_t disk_list;
+
+    /* Get list of disks */
+    if (IOServiceGetMatchingServices(kIOMasterPortDefault,
+                                     IOServiceMatching(kIOMediaClass),
+                                     &disk_list) != kIOReturnSuccess) {
+        Py_DECREF(py_retdict);
+        PyErr_SetString(PyExc_RuntimeError, "Unable to get the list of disks.");
+        return NULL;
+    }
+
+    /* Iterate over disks */
+    while ((disk = IOIteratorNext(disk_list)) != 0) {
+        parent_dict = NULL;
+        props_dict = NULL;
+        stats_dict = NULL;
+
+        if (IORegistryEntryGetParentEntry(disk, kIOServicePlane, &parent) != kIOReturnSuccess) {
+            PyErr_SetString(PyExc_RuntimeError, "Unable to get the disk's parent.");
+            Py_DECREF(py_retdict);
+            IOObjectRelease(disk);
+            return NULL;
+        }
+
+        if (IOObjectConformsTo(parent, "IOBlockStorageDriver")) {
+            if(IORegistryEntryCreateCFProperties(
+                                     disk,
+                                     (CFMutableDictionaryRef *) &parent_dict,
+                                     kCFAllocatorDefault,
+                                     kNilOptions) != kIOReturnSuccess)
+            {
+                PyErr_SetString(PyExc_RuntimeError,
+                                "Unable to get the parent's properties.");
+                Py_DECREF(py_retdict);
+                IOObjectRelease(disk);
+                IOObjectRelease(parent);
+                return NULL;
+            }
+
+            if (IORegistryEntryCreateCFProperties(parent,
+                                          (CFMutableDictionaryRef *) &props_dict,
+                                          kCFAllocatorDefault,
+                                          kNilOptions) != kIOReturnSuccess)
+            {
+                PyErr_SetString(PyExc_RuntimeError,
+                                "Unable to get the disk properties.");
+                Py_DECREF(py_retdict);
+                IOObjectRelease(disk);
+                return NULL;
+            }
+
+            const int kMaxDiskNameSize = 64;
+            CFStringRef disk_name_ref = (CFStringRef)CFDictionaryGetValue(
+                                                    parent_dict,
+                                                    CFSTR(kIOBSDNameKey));
+            char disk_name[kMaxDiskNameSize];
+
+            CFStringGetCString(disk_name_ref,
+                               disk_name,
+                               kMaxDiskNameSize,
+                               CFStringGetSystemEncoding());
+
+            stats_dict = (CFDictionaryRef)CFDictionaryGetValue(
+                                    props_dict,
+                                    CFSTR(kIOBlockStorageDriverStatisticsKey));
+
+            if (stats_dict == NULL) {
+                PyErr_SetString(PyExc_RuntimeError, "Unable to get disk stats.");
+                Py_DECREF(py_retdict);
+                CFRelease(props_dict);
+                IOObjectRelease(disk);
+                IOObjectRelease(parent);
+                return NULL;
+            }
+
+            CFNumberRef number;
+            int64_t reads, writes, read_bytes, write_bytes, read_time, write_time = 0;
+
+            /* Get disk reads/writes */
+            if ((number = (CFNumberRef)CFDictionaryGetValue(
+                            stats_dict,
+                            CFSTR(kIOBlockStorageDriverStatisticsReadsKey))))
+            {
+                CFNumberGetValue(number, kCFNumberSInt64Type, &reads);
+            }
+            if ((number = (CFNumberRef)CFDictionaryGetValue(
+                            stats_dict,
+                            CFSTR(kIOBlockStorageDriverStatisticsWritesKey))))
+            {
+                CFNumberGetValue(number, kCFNumberSInt64Type, &writes);
+            }
+
+            /* Get disk bytes read/written */
+            if ((number = (CFNumberRef)CFDictionaryGetValue(
+                        stats_dict,
+                        CFSTR(kIOBlockStorageDriverStatisticsBytesReadKey))))
+            {
+                CFNumberGetValue(number, kCFNumberSInt64Type, &read_bytes);
+            }
+            if ((number = (CFNumberRef)CFDictionaryGetValue(
+                stats_dict,
+                CFSTR(kIOBlockStorageDriverStatisticsBytesWrittenKey))))
+            {
+                CFNumberGetValue(number, kCFNumberSInt64Type, &write_bytes);
+            }
+
+            /* Get disk time spent reading/writing (nanoseconds) */
+            if ((number = (CFNumberRef)CFDictionaryGetValue(
+                    stats_dict,
+                    CFSTR(kIOBlockStorageDriverStatisticsTotalReadTimeKey))))
+            {
+                CFNumberGetValue(number, kCFNumberSInt64Type, &read_time);
+            }
+            if ((number = (CFNumberRef)CFDictionaryGetValue(
+                    stats_dict,
+                    CFSTR(kIOBlockStorageDriverStatisticsTotalWriteTimeKey)))) {
+                CFNumberGetValue(number, kCFNumberSInt64Type, &write_time);
+            }
+
+            // Read/Write time on OS X comes back in nanoseconds and in psutil
+            // we've standardized on milliseconds so do the conversion.
+            py_disk_info = Py_BuildValue("(KKKKKK)",
+                                         reads, writes,
+                                         read_bytes, write_bytes,
+                                         read_time / 1000, write_time / 1000);
+            PyDict_SetItemString(py_retdict, disk_name, py_disk_info);
+            Py_XDECREF(py_disk_info);
+
+            CFRelease(parent_dict);
+            IOObjectRelease(parent);
+            CFRelease(props_dict);
+            IOObjectRelease(disk);
+        }
+    }
+
+    IOObjectRelease (disk_list);
+
+    return py_retdict;
 }
 
 
@@ -1114,6 +1391,10 @@ PsutilMethods[] =
      {"get_disk_partitions", get_disk_partitions, METH_VARARGS,
          "Return a list of tuples including device, mount point and "
          "fs type for all partitions mounted on the system."},
+     {"get_network_io_counters", get_network_io_counters, METH_VARARGS,
+         "Return dict of tuples of networks I/O information."},
+     {"get_disk_io_counters", get_disk_io_counters, METH_VARARGS,
+         "Return dict of tuples of disks I/O information."},
 
      {NULL, NULL, 0, NULL}
 };
@@ -1189,5 +1470,3 @@ init_psutil_osx(void)
     return module;
 #endif
 }
-
-
