@@ -1,6 +1,10 @@
 /*
  * $Id$
  *
+ * Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
+ *
  * Helper functions related to fetching process information. Used by _psutil_osx
  * module methods.
  */
@@ -119,10 +123,10 @@ get_proc_list(kinfo_proc **procList, size_t *procCount)
 
 
 /* Read the maximum argument size for processes */
-size_t
+int
 get_argmax()
 {
-    size_t argmax;
+    int argmax;
     int mib[] = { CTL_KERN, KERN_ARGMAX };
     size_t size = sizeof(argmax);
 
