@@ -2,6 +2,11 @@
 #
 # $Id$
 #
+# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+# Use of this source code is governed by a BSD-style license that can be
+# found in the LICENSE file.
+
+"""POSIX specific tests.  These are implicitly run by test_psutil.py."""
 
 import unittest
 import subprocess
@@ -86,8 +91,8 @@ class PosixSpecificTestCase(unittest.TestCase):
         # use command + arg since "comm" keyword not supported on all platforms
         name_ps = ps("ps --no-headers -o command -p %s" %self.pid).split(' ')[0]
         # remove path if there is any, from the command
-        name_ps = os.path.basename(name_ps)
-        name_psutil = psutil.Process(self.pid).name
+        name_ps = os.path.basename(name_ps).lower()
+        name_psutil = psutil.Process(self.pid).name.lower()
         self.assertEqual(name_ps, name_psutil)
 
     def test_process_exe(self):
@@ -125,19 +130,30 @@ class PosixSpecificTestCase(unittest.TestCase):
                 pids_ps.append(int(pid.strip()))
         # remove ps subprocess pid which is supposed to be dead in meantime
         pids_ps.remove(p.pid)
-        # not all systems include pid 0 in their list but psutil does so
-        # we force it
-        if 0 not in pids_ps:
-            pids_ps.append(0)
-
         pids_psutil = psutil.get_pid_list()
         pids_ps.sort()
         pids_psutil.sort()
 
+        # on OSX ps doesn't show pid 0
+        if OSX and 0 not in pids_ps:
+            pids_ps.insert(0, 0)
+
         if pids_ps != pids_psutil:
             difference = [x for x in pids_psutil if x not in pids_ps] + \
                          [x for x in pids_ps if x not in pids_psutil]
             self.fail("difference: " + str(difference))
+            
+    def test_nic_names(self):
+        p = subprocess.Popen("ifconfig -a", shell=1, stdout=subprocess.PIPE)
+        output = p.communicate()[0].strip()
+        if sys.version_info >= (3,):
+            output = str(output, sys.stdout.encoding)
+        for nic in psutil.network_io_counters(pernic=True).keys():
+            for line in output.split():
+                if line.startswith(nic):
+                    break
+            else:
+                self.fail("couldn't find %s nic in 'ifconfig -a' output" % nic)
 
 
 if __name__ == '__main__':
