@@ -2,22 +2,24 @@
 #
 # $Id$
 #
+# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+# Use of this source code is governed by a BSD-style license that can be
+# found in the LICENSE file.
 
 """
-Note: this is targeted for python 2.x.
-To run it under python 3.x you need to use 2to3 tool first:
-
-$ 2to3 -w test/test_memory_leaks.py
+A test script which attempts to detect memory leaks by calling C
+functions many times and compare process memory usage before and
+after the calls.  It might produce false positives.
 """
 
-
 import os
 import gc
 import unittest
 import time
+import socket
 
 import psutil
-from test_psutil import reap_children, skipUnless, skipIf, \
+from test_psutil import reap_children, skipUnless, skipIf, supports_ipv6, \
                         POSIX, LINUX, WINDOWS, OSX, BSD, PY3
 
 LOOPS = 1000
@@ -25,6 +27,10 @@ TOLERANCE = 4096
 
 if PY3:
     xrange = range
+try:
+    callable
+except NameError:
+    callable = lambda x: hasattr(x, '__call__')
 
 
 class Base(unittest.TestCase):
@@ -140,17 +146,35 @@ class TestProcessObjectLeaks(Base):
     def test_resume(self):
         self.execute('resume')
 
-    @skipUnless(WINDOWS)
+    @skipIf(not hasattr(psutil.Process, 'getcwd'))
     def test_getcwd(self):
         self.execute('getcwd')
 
-    @skipUnless(WINDOWS or OSX)
     def test_get_open_files(self):
         self.execute('get_open_files')
 
-    @skipUnless(WINDOWS or OSX)
+    # XXX - BSD still uses provisional lsof implementation
+    # Linux implementation is pure python so since it's slow we skip it
+    @skipIf(BSD or LINUX)
     def test_get_connections(self):
-        self.execute('get_connections')
+        def create_socket(family, type):
+            sock = socket.socket(family, type)
+            sock.bind(('', 0))
+            if type == socket.SOCK_STREAM:
+                sock.listen(1)
+            return sock
+
+        socks = []
+        socks.append(create_socket(socket.AF_INET, socket.SOCK_STREAM))
+        socks.append(create_socket(socket.AF_INET, socket.SOCK_DGRAM))
+        if supports_ipv6():
+            socks.append(create_socket(socket.AF_INET6, socket.SOCK_STREAM))
+            socks.append(create_socket(socket.AF_INET6, socket.SOCK_DGRAM))
+        try:
+            self.execute('get_connections', kind='all')
+        finally:
+            for s in socks:
+                s.close()
 
 
 class TestModuleFunctionsLeaks(Base):
@@ -193,6 +217,11 @@ class TestModuleFunctionsLeaks(Base):
     def test_disk_partitions(self):
         self.execute('disk_partitions')
 
+    def test_network_io_counters(self):
+        self.execute('network_io_counters')
+
+    def test_disk_io_counters(self):
+        self.execute('disk_io_counters')
 
 def test_main():
     test_suite = unittest.TestSuite()
@@ -202,4 +231,3 @@ def test_main():
 
 if __name__ == '__main__':
     test_main()
-
