@@ -2,12 +2,15 @@
 #
 # $Id$
 #
+# Copyright (c) 2009, Jay Loden, Giampaolo Rodola'. All rights reserved.
+# Use of this source code is governed by a BSD-style license that can be
+# found in the LICENSE file.
 
 """
 psutil test suite.
 
-Note: this is targeted for both python 2.x and 3.x.
-There's no need to use 2to3 first.
+Note: this is targeted for both python 2.x and 3.x so there's no need
+to use 2to3 tool first.
 """
 
 import unittest
@@ -165,6 +168,23 @@ def ignore_access_denied(fun):
         return inner
     return outer
 
+def supports_ipv6():
+    """Return True if IPv6 is supported on this platform."""
+    if not socket.has_ipv6 or not hasattr(socket, "AF_INET6"):
+        return False
+    sock = None
+    try:
+        try:
+            sock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
+            sock.bind(("::1", 0))
+        except (socket.error, socket.gaierror):
+            return False
+        else:
+            return True
+    finally:
+        if sock is not None:
+            sock.close()
+
 
 class ThreadTask(threading.Thread):
     """A thread object used for running process thread tests."""
@@ -215,12 +235,10 @@ class TestCase(unittest.TestCase):
     def test_get_process_list(self):
         pids = [x.pid for x in psutil.get_process_list()]
         self.assertTrue(os.getpid() in pids)
-        self.assertTrue(0 in pids)
 
     def test_process_iter(self):
         pids = [x.pid for x in psutil.process_iter()]
         self.assertTrue(os.getpid() in pids)
-        self.assertTrue(0 in pids)
 
     def test_TOTAL_PHYMEM(self):
         x = psutil.TOTAL_PHYMEM
@@ -421,6 +439,46 @@ class TestCase(unittest.TestCase):
         self.assertTrue(mount in mounts)
         psutil.disk_usage(mount)
 
+    def test_network_io_counters(self):
+        def check_ntuple(nt):
+            self.assertEqual(nt[0], nt.bytes_sent)
+            self.assertEqual(nt[1], nt.bytes_recv)
+            self.assertEqual(nt[2], nt.packets_sent)
+            self.assertEqual(nt[3], nt.packets_recv)
+            self.assertTrue(nt.bytes_sent >= 0)
+            self.assertTrue(nt.bytes_recv >= 0)
+            self.assertTrue(nt.packets_sent >= 0)
+            self.assertTrue(nt.packets_recv >= 0)
+
+        ret = psutil.network_io_counters(pernic=False)
+        check_ntuple(ret)
+        ret = psutil.network_io_counters(pernic=True)
+        self.assertTrue(ret != [])
+        for key in ret:
+            self.assertTrue(key)
+            check_ntuple(ret[key])
+
+    def test_disk_io_counters(self):
+        def check_ntuple(nt):
+            self.assertEqual(nt[0], nt.read_count)
+            self.assertEqual(nt[1], nt.write_count)
+            self.assertEqual(nt[2], nt.read_bytes)
+            self.assertEqual(nt[3], nt.write_bytes)
+            self.assertEqual(nt[4], nt.read_time)
+            self.assertEqual(nt[5], nt.write_time)
+            self.assertTrue(nt.read_count >= 0)
+            self.assertTrue(nt.write_count >= 0)
+            self.assertTrue(nt.read_bytes >= 0)
+            self.assertTrue(nt.write_bytes >= 0)
+            self.assertTrue(nt.read_time >= 0)
+            self.assertTrue(nt.write_time >= 0)
+
+        ret = psutil.disk_io_counters(perdisk=False)
+        check_ntuple(ret)
+        ret = psutil.disk_io_counters(perdisk=True)
+        for key in ret:
+            self.assertTrue(key)
+            check_ntuple(ret[key])
 
     # ====================
     # Process object tests
@@ -503,6 +561,9 @@ class TestCase(unittest.TestCase):
         p.name
         self.assertRaises(psutil.TimeoutExpired, p.wait, 0.01)
 
+        # timeout < 0 not allowed
+        self.assertRaises(ValueError, p.wait, -1)
+
     @skipUnless(POSIX)
     def test_wait_non_children(self):
         # test wait() against processes which are not our children
@@ -525,6 +586,26 @@ class TestCase(unittest.TestCase):
                 grandson_proc.kill()
                 grandson_proc.wait()
 
+    def test_wait_timeout_0(self):
+        sproc = get_test_subprocess()
+        p = psutil.Process(sproc.pid)
+        self.assertRaises(psutil.TimeoutExpired, p.wait, 0)
+        p.kill()
+        stop_at = time.time() + 2
+        while 1:
+            try:
+                code = p.wait(0)
+            except psutil.TimeoutExpired:
+                if time.time() >= stop_at:
+                    raise
+            else:
+                break
+        if os.name == 'posix':
+            self.assertEqual(code, signal.SIGKILL)
+        else:
+            self.assertEqual(code, 0)
+        self.assertFalse(p.is_running())
+
     def test_cpu_percent(self):
         p = psutil.Process(os.getpid())
         p.get_cpu_percent(interval=0.001)
@@ -533,7 +614,10 @@ class TestCase(unittest.TestCase):
             percent = p.get_cpu_percent(interval=None)
             self.assertTrue(isinstance(percent, float))
             self.assertTrue(percent >= 0.0)
-            self.assertTrue(percent <= 100.0)
+            if os.name != 'posix':
+                self.assertTrue(percent <= 100.0)
+            else:
+                self.assertTrue(percent >= 0.0)
 
     def test_cpu_times(self):
         times = psutil.Process(os.getpid()).get_cpu_times()
@@ -716,11 +800,6 @@ class TestCase(unittest.TestCase):
         sproc = get_test_subprocess()
         self.assertEqual(psutil.Process(sproc.pid).pid, sproc.pid)
 
-    def test_eq(self):
-        sproc = get_test_subprocess()
-        wait_for_pid(sproc.pid)
-        self.assertTrue(psutil.Process(sproc.pid) == psutil.Process(sproc.pid))
-
     def test_is_running(self):
         sproc = get_test_subprocess()
         wait_for_pid(sproc.pid)
@@ -768,8 +847,8 @@ class TestCase(unittest.TestCase):
     def test_name(self):
         sproc = get_test_subprocess(PYTHON)
         wait_for_pid(sproc.pid)
-        self.assertEqual(psutil.Process(sproc.pid).name,
-                         os.path.basename(sys.executable))
+        self.assertEqual(psutil.Process(sproc.pid).name.lower(),
+                         os.path.basename(sys.executable).lower())
 
     if os.name == 'posix':
 
@@ -800,18 +879,22 @@ class TestCase(unittest.TestCase):
         def test_nice(self):
             p = psutil.Process(os.getpid())
             self.assertRaises(TypeError, setattr, p, "nice", "str")
-            first_nice = p.nice
             try:
-                p.nice = 1
-                self.assertEqual(p.nice, 1)
-                # going back to previous nice value raises AccessDenied on OSX
-                if not OSX:
-                    p.nice = 0
-                    self.assertEqual(p.nice, 0)
+                try:
+                    first_nice = p.nice
+                    p.nice = 1
+                    self.assertEqual(p.nice, 1)
+                    # going back to previous nice value raises AccessDenied on OSX
+                    if not OSX:
+                        p.nice = 0
+                        self.assertEqual(p.nice, 0)
+                except psutil.AccessDenied:
+                    pass
             finally:
-                # going back to previous nice value raises AccessDenied on OSX
-                if not OSX:
+                try:
                     p.nice = first_nice
+                except psutil.AccessDenied:
+                    pass
 
     if os.name == 'nt':
 
@@ -850,14 +933,14 @@ class TestCase(unittest.TestCase):
         else:
             p.username
 
-    @skipUnless(WINDOWS or LINUX)
+    @skipIf(not hasattr(psutil.Process, "getcwd"))
     def test_getcwd(self):
         sproc = get_test_subprocess()
         wait_for_pid(sproc.pid)
         p = psutil.Process(sproc.pid)
         self.assertEqual(p.getcwd(), os.getcwd())
 
-    @skipUnless(WINDOWS or LINUX)
+    @skipIf(not hasattr(psutil.Process, "getcwd"))
     def test_getcwd_2(self):
         cmd = [PYTHON, "-c", "import os, time; os.chdir('..'); time.sleep(10)"]
         sproc = get_test_subprocess(cmd)
@@ -961,11 +1044,23 @@ class TestCase(unittest.TestCase):
         self.assertEqual(con[3], con.local_address)
         self.assertEqual(con[4], con.remote_address)
         self.assertEqual(con[5], con.status)
+        # test kind arg
+        self.assertRaises(ValueError, p.get_connections, 'foo')
+
+    @skipUnless(supports_ipv6())
+    def test_get_connections_ipv6(self):
+        s = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
+        s.bind(('::1', 0))
+        s.listen(1)
+        cons = psutil.Process(os.getpid()).get_connections()
+        s.close()
+        self.assertEqual(len(cons), 1)
+        self.assertEqual(cons[0].local_address[0], '::1')
 
     @skipUnless(hasattr(socket, "fromfd") and not WINDOWS)
     def test_connection_fromfd(self):
         sock = socket.socket()
-        sock.bind(('127.0.0.1', 0))
+        sock.bind(('localhost', 0))
         sock.listen(1)
         p = psutil.Process(os.getpid())
         for conn in p.get_connections():
@@ -997,22 +1092,6 @@ class TestCase(unittest.TestCase):
                     self.assertTrue(0 <= num <= 255)
             self.assertTrue(0 <= port <= 65535)
 
-        def supports_ipv6():
-            if not socket.has_ipv6 or not hasattr(socket, "AF_INET6"):
-                return False
-            sock = None
-            try:
-                try:
-                    sock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
-                    sock.bind(("::1", 0))
-                except (socket.error, socket.gaierror):
-                    return False
-                else:
-                    return True
-            finally:
-                if sock is not None:
-                    sock.close()
-
         # all values are supposed to match Linux's tcp_states.h states
         # table across all platforms.
         valid_states = ["ESTABLISHED", "SYN_SENT", "SYN_RECV", "FIN_WAIT1",
@@ -1041,7 +1120,7 @@ class TestCase(unittest.TestCase):
                                                           addr="::1")
 
         # launch various subprocess instantiating a socket of various
-        # families  and tupes to enrich psutil results
+        # families and types to enrich psutil results
         tcp4_proc = get_test_subprocess([PYTHON, "-c", tcp4_template])
         udp4_proc = get_test_subprocess([PYTHON, "-c", udp4_template])
         if supports_ipv6():
@@ -1101,6 +1180,9 @@ class TestCase(unittest.TestCase):
 
         # --- check matches against subprocesses
 
+        all_kinds = ("all", "inet", "inet4", "inet6", "tcp", "tcp4", "tcp6",
+                     "udp", "udp4", "udp6")
+
         for p in psutil.Process(os.getpid()).get_children():
             for conn in p.get_connections():
                 # TCP v4
@@ -1110,6 +1192,12 @@ class TestCase(unittest.TestCase):
                     self.assertEqual(conn.local_address[0], "127.0.0.1")
                     self.assertEqual(conn.remote_address, ())
                     self.assertEqual(conn.status, "LISTEN")
+                    for kind in all_kinds:
+                        cons = p.get_connections(kind=kind)
+                        if kind in ("all", "inet", "inet4", "tcp", "tcp4"):
+                            self.assertTrue(cons != [])
+                        else:
+                            self.assertEqual(cons, [])
                 # UDP v4
                 elif p.pid == udp4_proc.pid:
                     self.assertEqual(conn.family, socket.AF_INET)
@@ -1117,6 +1205,12 @@ class TestCase(unittest.TestCase):
                     self.assertEqual(conn.local_address[0], "127.0.0.1")
                     self.assertEqual(conn.remote_address, ())
                     self.assertEqual(conn.status, "")
+                    for kind in all_kinds:
+                        cons = p.get_connections(kind=kind)
+                        if kind in ("all", "inet", "inet4", "udp", "udp4"):
+                            self.assertTrue(cons != [])
+                        else:
+                            self.assertEqual(cons, [])
                 # TCP v6
                 elif p.pid == getattr(tcp6_proc, "pid", None):
                     self.assertEqual(conn.family, socket.AF_INET6)
@@ -1124,6 +1218,12 @@ class TestCase(unittest.TestCase):
                     self.assertTrue(conn.local_address[0] in ("::", "::1"))
                     self.assertEqual(conn.remote_address, ())
                     self.assertEqual(conn.status, "LISTEN")
+                    for kind in all_kinds:
+                        cons = p.get_connections(kind=kind)
+                        if kind in ("all", "inet", "inet6", "tcp", "tcp6"):
+                            self.assertTrue(cons != [])
+                        else:
+                            self.assertEqual(cons, [])
                 # UDP v6
                 elif p.pid == getattr(udp6_proc, "pid", None):
                     self.assertEqual(conn.family, socket.AF_INET6)
@@ -1131,6 +1231,12 @@ class TestCase(unittest.TestCase):
                     self.assertTrue(conn.local_address[0] in ("::", "::1"))
                     self.assertEqual(conn.remote_address, ())
                     self.assertEqual(conn.status, "")
+                    for kind in all_kinds:
+                        cons = p.get_connections(kind=kind)
+                        if kind in ("all", "inet", "inet6", "udp", "udp6"):
+                            self.assertTrue(cons != [])
+                        else:
+                            self.assertEqual(cons, [])
 
     def test_parent_ppid(self):
         this_parent = os.getpid()
@@ -1215,7 +1321,9 @@ class TestCase(unittest.TestCase):
         sproc = get_test_subprocess()
         p = psutil.Process(sproc.pid)
         self.assertTrue(str(sproc.pid) in str(p))
-        self.assertTrue(os.path.basename(PYTHON) in str(p))
+        # python shows up as 'Python' in cmdline on OS X so test fails on OS X
+        if not OSX:
+            self.assertTrue(os.path.basename(PYTHON) in str(p))
         sproc = get_test_subprocess()
         p = psutil.Process(sproc.pid)
         p.kill()
@@ -1230,8 +1338,6 @@ class TestCase(unittest.TestCase):
         excluded_names += ['get_cpu_percent', 'get_children']
         # XXX - skip slow lsof implementation;
         if BSD:
-           excluded_names += ['get_open_files', 'get_connections']
-        if OSX:
            excluded_names += ['get_connections']
         attrs = []
         for name in dir(psutil.Process):
@@ -1263,6 +1369,10 @@ class TestCase(unittest.TestCase):
                     else:
                         if name == 'parent' or ret in (0, 0.0, [], None):
                             continue
+                        # getcwd() on FreeBSD may be an empty string
+                        # in case of a system process
+                        if name == 'getcwd' and BSD and ret == '':
+                            continue
                         self.assertTrue(ret)
                         if name == "exe":
                             self.assertTrue(os.path.isfile(ret))
@@ -1283,9 +1393,9 @@ class TestCase(unittest.TestCase):
         # special cases.
         self.assertTrue(valid_procs > 0)
 
+    @skipIf(LINUX)
     def test_pid_0(self):
-        # Process(0) is supposed to work on all platforms even if with
-        # some differences
+        # Process(0) is supposed to work on all platforms except Linux
         p = psutil.Process(0)
         self.assertTrue(p.name)
 
@@ -1315,12 +1425,14 @@ class TestCase(unittest.TestCase):
         else:
             p.username
 
-        # PID 0 is supposed to be available on all platforms
         self.assertTrue(0 in psutil.get_pid_list())
         self.assertTrue(psutil.pid_exists(0))
 
     def test_Popen(self):
         # Popen class test
+        # XXX this test causes a ResourceWarning on Python 3 because
+        # psutil.__subproc instance doesn't get propertly freed.
+        # Not sure what to do though.
         cmd = [PYTHON, "-c", "import time; time.sleep(3600);"]
         proc = psutil.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         try:
